@@ -12,6 +12,8 @@ use std::env;
 use std::fs;
 use std::path::{Path, PathBuf};
 use std::process::{Command, Stdio};
+use std::thread;
+use std::time::Duration;
 
 // note that this list must be topologically sorted by dependencies
 const CRATES_TO_PUBLISH: &[&str] = &[
@@ -45,12 +47,10 @@ const CRATES_TO_PUBLISH: &[&str] = &[
     "wiggle-generate",
     "wiggle-macro",
     // wasmtime
-    "lightbeam",
     "wasmtime-fiber",
     "wasmtime-environ",
     "wasmtime-runtime",
     "wasmtime-cranelift",
-    "wasmtime-lightbeam",
     "wasmtime-jit",
     "wasmtime-cache",
     "wasmtime",
@@ -63,17 +63,53 @@ const CRATES_TO_PUBLISH: &[&str] = &[
     "wasmtime-wasi",
     "wasmtime-wasi-nn",
     "wasmtime-wasi-crypto",
-    "wasmtime-rust-macro",
-    "wasmtime-rust",
     "wasmtime-wast",
     "wasmtime-cli",
 ];
 
+// Anything **not** mentioned in this array is required to have an `=a.b.c`
+// dependency requirement on it to enable breaking api changes even in "patch"
+// releases since everything not mentioned here is just an organizational detail
+// that no one else should rely on.
+const PUBLIC_CRATES: &[&str] = &[
+    // just here to appease the script because these are submodules of this
+    // repository.
+    "wasi-crypto",
+    "witx",
+    // these are actually public crates which we cannot break the API of in
+    // patch releases.
+    "wasmtime",
+    "wasmtime-wasi",
+    "wasmtime-wasi-nn",
+    "wasmtime-wasi-crypto",
+    "wasmtime-cli",
+    // all cranelift crates are considered "public" in that they can't
+    // have breaking API changes in patch releases
+    "cranelift-entity",
+    "cranelift-bforest",
+    "cranelift-codegen-shared",
+    "cranelift-codegen-meta",
+    "cranelift-codegen",
+    "cranelift-reader",
+    "cranelift-serde",
+    "cranelift-module",
+    "cranelift-preopt",
+    "cranelift-frontend",
+    "cranelift-wasm",
+    "cranelift-native",
+    "cranelift-object",
+    "cranelift-interpreter",
+    "cranelift",
+    "cranelift-jit",
+    // This is a dependency of cranelift crates and as a result can't break in
+    // patch releases as well
+    "wasmtime-types",
+];
+
 struct Crate {
     manifest: PathBuf,
     name: String,
     version: String,
-    next_version: String,
     publish: bool,
 }
 
@@ -91,9 +127,9 @@ fn main() {
     crates.sort_by_key(|krate| pos.get(&krate.name[..]));
 
     match &env::args().nth(1).expect("must have one argument")[..] {
-        "bump" => {
+        name @ "bump" | name @ "bump-patch" => {
             for krate in crates.iter() {
-                bump_version(&krate, &crates);
+                bump_version(&krate, &crates, name == "bump-patch");
             }
             // update the lock file
             assert!(Command::new("cargo")
@@ -104,9 +140,29 @@ fn main() {
         }
 
         "publish" => {
-            for krate in crates.iter() {
-                publish(&krate);
+            // We have so many crates to publish we're frequently either
+            // rate-limited or we run into issues where crates can't publish
+            // successfully because they're waiting on the index entries of
+            // previously-published crates to propagate. This means we try to
+            // publish in a loop and we remove crates once they're successfully
+            // published. Failed-to-publish crates get enqueued for another try
+            // later on.
+            for _ in 0..5 {
+                crates.retain(|krate| !publish(krate));
+
+                if crates.is_empty() {
+                    break;
+                }
+
+                println!(
+                    "{} crates failed to publish, waiting for a bit to retry",
+                    crates.len(),
+                );
+                thread::sleep(Duration::from_secs(20));
             }
+
+            assert!(crates.is_empty(), "failed to publish all crates");
+
             println!("");
             println!("===================================================================");
             println!("");
@@ -169,11 +225,6 @@ fn read_crate(manifest: &Path) -> Crate {
     }
     let name = name.unwrap();
     let version = version.unwrap();
-    let next_version = if CRATES_TO_PUBLISH.contains(&&name[..]) {
-        bump(&version)
-    } else {
-        version.clone()
-    };
     if ["witx", "witx-cli", "wasi-crypto"].contains(&&name[..]) {
         publish = false;
     }
@@ -181,13 +232,19 @@ fn read_crate(manifest: &Path) -> Crate {
         manifest: manifest.to_path_buf(),
         name,
         version,
-        next_version,
         publish,
     }
 }
 
-fn bump_version(krate: &Crate, crates: &[Crate]) {
+fn bump_version(krate: &Crate, crates: &[Crate], patch: bool) {
     let contents = fs::read_to_string(&krate.manifest).unwrap();
+    let next_version = |krate: &Crate| -> String {
+        if CRATES_TO_PUBLISH.contains(&&krate.name[..]) {
+            bump(&krate.version, patch)
+        } else {
+            krate.version.clone()
+        }
+    };
 
     let mut new_manifest = String::new();
     let mut is_deps = false;
@@ -197,9 +254,11 @@ fn bump_version(krate: &Crate, crates: &[Crate]) {
             if CRATES_TO_PUBLISH.contains(&&krate.name[..]) {
                 println!(
                     "bump `{}` {} => {}",
-                    krate.name, krate.version, krate.next_version
+                    krate.name,
+                    krate.version,
+                    next_version(krate),
                 );
-                new_manifest.push_str(&line.replace(&krate.version, &krate.next_version));
+                new_manifest.push_str(&line.replace(&krate.version, &next_version(krate)));
                 rewritten = true;
             }
         }
@@ -211,11 +270,17 @@ fn bump_version(krate: &Crate, crates: &[Crate]) {
         };
 
         for other in crates {
+            // If `other` isn't a published crate then it's not going to get a
+            // bumped version so we don't need to update anything in the
+            // manifest.
+            if !other.publish {
+                continue;
+            }
             if !is_deps || !line.starts_with(&format!("{} ", other.name)) {
                 continue;
             }
             if !line.contains(&other.version) {
-                if !line.contains("version =") {
+                if !line.contains("version =") || !krate.publish {
                     continue;
                 }
                 panic!(
@@ -223,8 +288,25 @@ fn bump_version(krate: &Crate, crates: &[Crate]) {
                     krate.manifest, other.name, other.version
                 );
             }
+            if krate.publish {
+                if PUBLIC_CRATES.contains(&other.name.as_str()) {
+                    assert!(
+                        !line.contains("\"="),
+                        "{} should not have an exact version requirement on {}",
+                        krate.name,
+                        other.name
+                    );
+                } else {
+                    assert!(
+                        line.contains("\"="),
+                        "{} should have an exact version requirement on {}",
+                        krate.name,
+                        other.name
+                    );
+                }
+            }
             rewritten = true;
-            new_manifest.push_str(&line.replace(&other.version, &other.next_version));
+            new_manifest.push_str(&line.replace(&other.version, &next_version(other)));
             break;
         }
         if !rewritten {
@@ -242,11 +324,15 @@ fn bump_version(krate: &Crate, crates: &[Crate]) {
 /// repository since we're currently making major version bumps for all our
 /// releases. This may end up getting tweaked as we stabilize crates and start
 /// doing more minor/patch releases, but for now this should do the trick.
-fn bump(version: &str) -> String {
+fn bump(version: &str, patch_bump: bool) -> String {
     let mut iter = version.split('.').map(|s| s.parse::<u32>().unwrap());
     let major = iter.next().expect("major version");
     let minor = iter.next().expect("minor version");
     let patch = iter.next().expect("patch version");
+
+    if patch_bump {
+        return format!("{}.{}.{}", major, minor, patch + 1);
+    }
     if major != 0 {
         format!("{}.0.0", major + 1)
     } else if minor != 0 {
@@ -256,10 +342,28 @@ fn bump(version: &str) -> String {
     }
 }
 
-fn publish(krate: &Crate) {
+fn publish(krate: &Crate) -> bool {
     if !CRATES_TO_PUBLISH.iter().any(|s| *s == krate.name) {
-        return;
+        return true;
     }
+
+    // First make sure the crate isn't already published at this version. This
+    // script may be re-run and there's no need to re-attempt previous work.
+    let output = Command::new("curl")
+        .arg(&format!("https://crates.io/api/v1/crates/{}", krate.name))
+        .output()
+        .expect("failed to invoke `curl`");
+    if output.status.success()
+        && String::from_utf8_lossy(&output.stdout)
+            .contains(&format!("\"newest_version\":\"{}\"", krate.version))
+    {
+        println!(
+            "skip publish {} because {} is latest version",
+            krate.name, krate.version,
+        );
+        return true;
+    }
+
     let status = Command::new("cargo")
         .arg("publish")
         .current_dir(krate.manifest.parent().unwrap())
@@ -268,18 +372,47 @@ fn publish(krate: &Crate) {
         .expect("failed to run cargo");
     if !status.success() {
         println!("FAIL: failed to publish `{}`: {}", krate.name, status);
+        return false;
+    }
+
+    // After we've published then make sure that the `wasmtime-publish` group is
+    // added to this crate for future publications. If it's already present
+    // though we can skip the `cargo owner` modification.
+    let output = Command::new("curl")
+        .arg(&format!(
+            "https://crates.io/api/v1/crates/{}/owners",
+            krate.name
+        ))
+        .output()
+        .expect("failed to invoke `curl`");
+    if output.status.success()
+        && String::from_utf8_lossy(&output.stdout).contains("wasmtime-publish")
+    {
+        println!(
+            "wasmtime-publish already listed as an owner of {}",
+            krate.name
+        );
+        return true;
     }
 
     // Note that the status is ignored here. This fails most of the time because
     // the owner is already set and present, so we only want to add this to
     // crates which haven't previously been published.
-    Command::new("cargo")
+    let status = Command::new("cargo")
         .arg("owner")
         .arg("-a")
         .arg("github:bytecodealliance:wasmtime-publish")
         .arg(&krate.name)
         .status()
         .expect("failed to run cargo");
+    if !status.success() {
+        panic!(
+            "FAIL: failed to add wasmtime-publish as owner `{}`: {}",
+            krate.name, status
+        );
+    }
+
+    true
 }
 
 // Verify the current tree is publish-able to crates.io. The intention here is
@@ -325,8 +458,7 @@ fn verify(crates: &[Crate]) {
             .arg("--manifest-path")
             .arg(&krate.manifest)
             .env("CARGO_TARGET_DIR", "./target");
-        if krate.name.contains("lightbeam")
-            || krate.name == "witx"
+        if krate.name == "witx"
             || krate.name.contains("wasi-nn")
             || krate.name.contains("peepmatic")
         {
