@@ -23,6 +23,34 @@ fn link_undefined() -> Result<()> {
     Ok(())
 }
 
+#[test]
+fn undefined_error_message_with_linking() {
+    let mut config = Config::new();
+    config.wasm_module_linking(true);
+    let engine = Engine::new(&config).unwrap();
+    let module = Module::new(
+        &engine,
+        r#"
+        (module
+            (import "foo" "bar" (func))
+        )
+    "#,
+    )
+    .unwrap();
+    let linker = Linker::new(&engine);
+    let mut store = Store::new(&engine, ());
+    assert!(linker
+        .instantiate(&mut store, &module)
+        .unwrap_err()
+        .to_string()
+        .contains("foo"));
+    assert!(linker
+        .instantiate(&mut store, &module)
+        .unwrap_err()
+        .to_string()
+        .contains("bar"));
+}
+
 #[test]
 fn link_twice_bad() -> Result<()> {
     let mut store = Store::<()>::default();
@@ -285,7 +313,7 @@ fn funcs_live_on_to_fight_another_day() -> Result<()> {
         assert_eq!(flag.load(SeqCst), 0);
         let mut store = Store::new(&engine, ());
         let func = linker.get(&mut store, "", Some("")).unwrap();
-        func.into_func().unwrap().call(&mut store, &[])?;
+        func.into_func().unwrap().call(&mut store, &[], &mut [])?;
         assert_eq!(flag.load(SeqCst), 0);
         Ok(())
     };
